const inquirer = require("inquirer");
// const util = require("util");
// const downloadGitRepo = require("download-git-repo");
const path = require("path");
const chalk = require("chalk");
const figlet = require("figlet");
const fs = require("fs-extra");
const spawn = require("cross-spawn");

const {
  outputError,
  outputSuccessText,
  outputWarmingText,
  write,
  setLoading,
  renderFiles,
} = require("./utils");

/**
 * options:
 * - appName
 * - appType // webApp | rnApp
 * - appId
 */
class Generator {
  appName = "";
  appType = "";
  appId = "";
  templatePath = "";
  targetDir = "";
  isForceOverwrite = true;
  baseInquirer = [
    {
      type: "input", //type： input, number, confirm, list, checkbox ...
      name: "appName", // key
      message: "App Name:", // msg show in terminal
    },
    {
      type: "list",
      name: "appType",
      message: "App Type:",
      default: "webApp",
      choices: ["webApp", "rnApp"],
    },
    {
      type: "input",
      name: "appId",
      message: "App ID:",
    },
  ];
  constructor(options) {
    console.log(`\r\n ${chalk.cyan("start create app")} \r\n`);
    const generator = Object.assign({}, options);
    this.appName = generator.name || "";
    this.isForceOverwrite = generator.force || false;
    this.init();
  }

  async init() {
    try {
      await this.inquireCreateOptions();
      // await this.downloadTemplate();
      await this.writeFile();
      await this.initProject();
      await this.installPackages();
      // this.success();
    } catch (error) {
      console.log(`\r\n create error! `, error);
    } finally {
      setLoading(false);
    }
  }

  async inquireCreateOptions() {
    await this.inquireAppName();
    outputSuccessText(`set name success: ${this.appName}`);
    await this.inquireAppType();
    outputSuccessText(`set type success: ${this.appType}`);
    await this.inquireAppId();
    outputSuccessText(`set appId success: ${this.appId}`);
  }

  async inquireAppName() {
    if (!this.appName) {
      await this.prompt(this.baseInquirer[0], async ({ appName }) => {
        if (!appName) {
          outputWarmingText(`appName should not be empty!`);
          await this.inquireAppName();
        } else {
          this.appName = appName;
          await this.setTargetDir();
        }
      });
    } else {
      await this.setTargetDir();
    }
  }

  async setTargetDir() {
    if (!this.appName) {
      return;
    }
    this.targetDir = path.join(process.cwd(), this.appName);
    if (fs.existsSync(this.targetDir)) {
      await this.inquireOverwrite();
    } else {
      fs.mkdirSync(this.targetDir);
    }
  }

  async inquireOverwrite() {
    if (this.isForceOverwrite) {
      return;
    }
    await this.prompt(
      {
        type: "confirm",
        name: "isOverwrite",
        message: outputWarmingText(
          `Target directory "${this.targetDir}" is not empty. Overwrite existing files and continue?`
        ),
      },
      ({ isOverwrite }) => {
        if (!isOverwrite) {
          console.log("\r\n create fail, exit! \r\n");
          process.exit();
        }
      }
    );
  }

  async inquireAppType() {
    if (!this.appType) {
      await this.prompt(this.baseInquirer[1], async ({ appType }) => {
        this.appType = appType;
      });
    }
  }

  async inquireAppId() {
    if (!this.appId) {
      await this.prompt(this.baseInquirer[2], async ({ appId }) => {
        if (!appId) {
          outputWarmingText(`appId should not be empty!`);
          await this.inquireAppId();
        } else {
          this.appId = appId;
        }
      });
    }
  }

  async installPackages() {
    const dependencies = this.getDependencies();
    // spawn("cd", [this.targetDir], {
    //   stdio: "inherit",
    // });

    const child = spawn(
      "yarn",
      ["add"].concat(dependencies).concat(["--cwd", this.targetDir]),
      {
        stdio: "inherit",
      }
    );

    child.on("close", (code) => {
      // run error
      if (code !== 0) {
        console.log(outputError("Install dependencies fail!"));
        fs.removeSync(this.targetDir);
        process.exit(1);
      } else {
        console.log(outputSuccessText("Install finished~"));
        this.success();
      }
    });

    // child.on('error', function)
  }

  getDependencies() {
    const packageInfo = require(`${this.targetDir}/package.json`);
    const dependencies = Object.assign(
      {},
      packageInfo.dependencies,
      packageInfo.devDependencies
    );

    return Object.keys(dependencies).map(
      (key) => `${key}@${dependencies[key]}`
    );
  }

  // async downloadTemplate() {
  // promisify downloadGitRepo
  // const asyncDownloadGitRepo = util.promisify(downloadGitRepo);
  //   console.log(
  //     "show downloadTemplate:",
  //     this.appName,
  //     this.appType,
  //     this.targetDir,
  //     path.resolve(process.cwd(), this.targetDir)
  //   );
  //   const templateUrl =
  //     this.appType === "webApp"
  //       ? "https://git.garena.com/seatalk/react-native/seatalk-web-app-hello-world.git"
  //       : "https://git.garena.com/seatalk/react-native/seatalk-rn-app-hello-world.git";

  //   // download the template
  //   await asyncDownloadGitRepo(
  //     `direct:${templateUrl}`,
  //     path.resolve(process.cwd(), this.targetDir),
  //     { clone: true }
  //   );
  // }

  async writeFile() {
    this.templatePath =
      this.appType === "webApp"
        ? path.join(__dirname, `templates/seatalk-web-app-hello-world`)
        : path.join(__dirname, `templates/seatalk-rn-app-hello-world`);
    const files = fs.readdirSync(this.templatePath);
    for (const file of files.filter(
      // skip copy
      (f) => !["package.json", ".DS_Store", "MyApp.ts", ".git"].includes(f)
    )) {
      write(file, this.templatePath, this.targetDir);
    }
  }

  async initProject() {
    const { templatePath, appId, targetDir, appName } = this;
    // render file
    if (this.appType === "rnApp") {
      await renderFiles([
        {
          sourcePath: templatePath,
          targetPath: targetDir,
          name: "MyApp.ts",
          data: { appId },
        },
        {
          sourcePath: templatePath,
          targetPath: targetDir,
          name: "package.json",
          data: { appName },
        },
      ]);
    } else {
      await renderFiles([
        {
          sourcePath: templatePath,
          targetPath: targetDir,
          name: "package.json",
          data: { appName },
        },
      ]);
    }
  }

  success() {
    // use figlet to make Logo
    console.log(
      "\r\n" +
        figlet.textSync(`seatalk ${this.appType}`, {
          font: "Ghost",
          horizontalLayout: "default",
          verticalLayout: "default",
          width: 140,
          whitespaceBreak: true,
        })
    );
    console.log(
      `\r\n Create ${
        this.appType
      } successfully!  You can run commands as follows to start app: \r\n 
      ${chalk.cyan("cd " + this.appName)} \r\n    
      ${chalk.cyan("yarn start")} \r\n `
    );
  }

  async prompt(messageInfo, callback) {
    if (!messageInfo) {
      return false;
    }
    let res;
    if (Array.isArray(messageInfo) && messageInfo.length > 0) {
      res = await inquirer.prompt(messageInfo);
    } else if (typeof messageInfo === "object") {
      res = await inquirer.prompt(messageInfo);
    }
    if (callback) {
      return callback(res);
    }
  }
}

module.exports = Generator;
