import { Assets } from '@seatalk-rn/sdk-assets'
import { LocalImage, RemoteImage } from '@seatalk-rn/sdk-client/dist/share'
import { PageComponent, SafeAreaInsets, StyleSheet, View } from '@seatalk-rn/sdk-components'
import { Config } from '@seatalk-rn/sdk-config'
import { FileInfo, IFileManager } from '@seatalk-rn/sdk-filesystem'
import { getLastKnownLocation, LocationErrorCodes } from '@seatalk-rn/sdk-location'
import { Scanner } from '@seatalk-rn/sdk-scanner'
import { ShareAppLinkErrorCodes } from '@seatalk-rn/sdk-share'
import { UserModule } from '@seatalk-rn/sdk-user'
import { Alert, HUD, ImagePicker, ImagePickerError, NavBarPage, SolidButton } from '@seatalk-rn/sdk-widgets'
import { IAlertTextInputConfig } from '@seatalk-rn/sdk-widgets/dist/alert/AlertInputConfig'
import React from 'react'
import { ScrollView } from 'react-native'
import { sdkClient } from '../client'
import { TXT } from '../i18n'

interface IDemoState {
    pageInsets: SafeAreaInsets
}

export class DemoPage extends PageComponent<{}, IDemoState> {
    private logger = sdkClient.then(client => client.buildLogger('DemoPage'))

    async componentDidMount() {
        super.componentDidMount();
        (await this.logger).info('componentDidMount => DemoPage');
        (await this.logger).info(`pageInsets: ${JSON.stringify(this.pageInsets())}`)
    }

    async pageInsetsDidChange(newInsets: SafeAreaInsets) {
        (await this.logger).info(`pageInsetsUpdated: ${JSON.stringify(newInsets)}`, undefined)
    }

    getPageName(): string {
        return 'DemoPage'
    }

    async onPressPush() {
        await (await sdkClient).navigator.push('/', {}, { pushType: 'normal' })
    }

    async onPressPresent() {
        await (await sdkClient).navigator.push('/', {}, { pushType: 'modal' })
    }

    onPressConfig() {
        HUD.hudShowSuccess(
            `Env: "${Config.env}" Lang: "${Config.languageCode}" DeviceID: "${Config.deviceID}" DeviceModel: "${Config.deviceModel}"`
        )
    }

    async onPressAuth() {
        HUD.hudShowLoading()
        try {
            const token = await (await sdkClient).openPlatform.fetchAuthToken()
            HUD.hudShowSuccess(`Token ${token}`)
        } catch (error) {
            HUD.hudShowError(`Error: ${JSON.stringify(error)}`)
        }
    }

    async onPressScanner() {
        try {
            const result = await Scanner.scanQRCode({ supportedTypes: ['user'] }, { pushType: 'modal' })
            if (!result.wasCancelled) {
                HUD.hudShowSuccess(`UserID: ${result.payload}`)
            }
            (await this.logger).info(`WasCancelled: ${result.wasCancelled}`)
        } catch (error) {
            (await this.logger).error(`Error: ${error}`)
        }
    }

    onPressTextNavBarMenu() {
        HUD.hudShowSuccess('Tapped')
    }

    async onPressStoreTest() {
        const store = (await sdkClient).storage.createStore()
        let title = await store.get('alert_title')
        if (title === null) {
            title = 'No Title Set'
        }

        await Alert.show(
            title,
            'Try changing the title',
            {
                title: 'Confirm',
                style: 'normal',
                callback: async input => {
                    try {
                        if (input) {
                            await store.set('alert_title', input)
                        } else {
                            await store.set('alert_title', 'No Title Set')
                        }
                        HUD.hudShowSuccess('Title Changed!')
                    } catch (e) {
                        HUD.hudShowError(`Title couldn't be changed ${e}`)
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => HUD.hudShowSuccess('Cancelled'),
            },
            {
                placeholder: 'Enter your title..',
            }
        )
    }

    async onPressComponentsTest() {
        await (await sdkClient).navigator.push('/components')
    }

    async onPressCalendarTest() {
        await (await sdkClient).navigator.push('/calendar')
    }

    async onPressNavigateToUserProfile() {
        try {
            const result = await Scanner.scanQRCode({ supportedTypes: ['user'] }, { pushType: 'normal' });
            (await this.logger).info(`WasCancelled: ${result.wasCancelled}`)
            if (!result.wasCancelled) {
                await UserModule.navigateToUserProfile(result.payload)
            }
        } catch (error) {
            (await this.logger).error(`Error: ${error}`)
        }
    }

    async onPressWebView() {
        await (await sdkClient).navigator.push('/webview')
    }

    async onPressImagePicker(): Promise<void> {
        HUD.hudShowLoading()
        try {
            const result = await ImagePicker.selectImages(5)
            HUD.hudHide()
            if (!result.wasCancelled) {
                (await this.logger).info(`onPressImagePicker success: ${JSON.stringify(result)}`)
                await (await sdkClient).navigator.push('/imageviewer', { images: result.images })
            } else {
                HUD.hudShowError('Cancelled');
                (await this.logger).info('onPressImagePicker cancelled')
            }
        } catch (error) {
            HUD.hudHide();
            (await this.logger).error(error)
            switch (error.code) {
                case ImagePickerError.SDK_ERROR: {
                    HUD.hudShowError(`ImagePicker: SDK_ERROR, ${error.message}`)
                    break
                }
                case ImagePickerError.NO_PERMISSION: {
                    HUD.hudShowError(`ImagePicker: NO_PERMISSION, ${error.message}`)
                    break
                }
                default:
                    HUD.hudShowError(`ImagePicker: ${error.code}, ${error.message}`)
            }
        }
    }

    private async showCancellableInputDialog(
        title: string,
        inputConfig: IAlertTextInputConfig = {}
    ): Promise<{ cancelled: false; input: string | undefined } | { cancelled: true }> {
        let text: string | undefined = undefined
        let cancelled = false
        await Alert.show(
            title,
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => (text = input),
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => (cancelled = true),
            },
            inputConfig
        )
        if (cancelled) {
            return { cancelled }
        } else {
            return { cancelled, input: text }
        }
    }

    async onPressShareAppLink(): Promise<void> {
        try {
            let title: string
            let subtitle: string | undefined = undefined
            let path: string
            let params: Record<string, string> | undefined = undefined
            let imageUrl: string | undefined = undefined
            let download = false

            let result = await this.showCancellableInputDialog('Enter title')
            if (result.cancelled) {
                HUD.hudShowSuccess('Cancelled')
                return
            } else if (!result.input) {
                HUD.hudShowError('Invalid title')
                return
            } else {
                title = result.input
            }

            result = await this.showCancellableInputDialog('Enter subtitle')
            if (result.cancelled) {
                HUD.hudShowSuccess('Cancelled')
                return
            } else {
                subtitle = result.input
            }

            result = await this.showCancellableInputDialog('Enter path of the app (starting with "/")')
            if (result.cancelled) {
                HUD.hudShowSuccess('Cancelled')
                return
            } else if (!result.input || !result.input.startsWith('/')) {
                HUD.hudShowError('Invalid path')
                return
            } else {
                path = result.input
            }

            result = await this.showCancellableInputDialog('Enter params (format: key1=value1&key2=value2)')
            if (result.cancelled) {
                HUD.hudShowSuccess('Cancelled')
                return
            } else {
                params = result.input?.split('&').reduce((previousValue, currentValue) => {
                    const [key, value] = currentValue.split('=')
                    return { ...previousValue, [key]: value }
                }, {})
            }

            result = await this.showCancellableInputDialog('Enter Icon Image URL', {
                prefilledText:
                    'https://is2-ssl.mzstatic.com/image/thumb/Purple114/v4/4f/17/73/4f177329-378a-87ac-fad2-8e4a6dc302f3' +
                    '/AppIcon-0-0-1x_U007emarketing-0-0-0-4-0-0-sRGB-0-0-0-GLES2_U002c0-512MB-85-220-0-0.png/230x0w.png',
            })
            if (result.cancelled) {
                HUD.hudShowSuccess('Cancelled')
                return
            } else {
                imageUrl = result.input
            }

            await Alert.show(
                'Download the image or use remote URL directly?',
                'Yes: Download the image; No: Use remote URL',
                {
                    title: 'Yes',
                    style: 'normal',
                    callback: () => (download = true),
                },
                {
                    title: 'No',
                    style: 'normal',
                    callback: () => (download = false),
                }
            )

            let image: LocalImage | RemoteImage | undefined = undefined
            if (download && imageUrl) {
                const fileManager: IFileManager = (await sdkClient).fileSystem.fileManager
                const imageInfo: FileInfo = { type: 'cache', path: 'AppLinkIcon.png' }
                HUD.hudShowLoading()
                if (!(await fileManager.exists(imageInfo))) {
                    await fileManager.downloadFile(imageUrl, imageInfo, undefined)
                }
                HUD.hudHide()
                image = { type: 'local', image: imageInfo }
            } else if (imageUrl) {
                image = { type: 'remote', imageUrl }
            }
            await (await sdkClient).share.shareOpenPlatformAppLink({
                title,
                subtitle,
                image,
                path,
                params,
            })
        } catch (error) {
            switch (error.code) {
                case ShareAppLinkErrorCodes.SDK_ERROR:
                    HUD.hudShowError(`ShareAppLink: SDK_ERROR, ${error.message}`)
                    break
                default:
                    HUD.hudShowError(`ShareAppLink: ${error.code}, ${error.message}`)
            }
        }
    }

    async onPressFileSystemTest() {
        await (await sdkClient).navigator.push('/filesystem')
    }

    async onPressFilePreviewTest() {
        await (await sdkClient).navigator.push('/filepreview')
    }

    onPressPropsTest() {
        if (this.props) {
            const propsString = JSON.stringify(this.props)
            HUD.hudShowSuccess(propsString)
        } else {
            HUD.hudShowSuccess('No props found')
        }
    }

    async onPressGetLastKnownLocationTest() {
        try {
            const location = await getLastKnownLocation()
            HUD.hudShowSuccess(JSON.stringify(location))
        } catch (error) {
            switch (error.code) {
                case LocationErrorCodes.SDK_ERROR:
                    HUD.hudShowError(`GetLastKnownLocation: SDK_ERROR, ${error.message}`)
                    break
                case LocationErrorCodes.NO_PERMISSION:
                    HUD.hudShowError(`GetLastKnownLocation: NO_PERMISSION, ${error.message}`)
                    break
                case LocationErrorCodes.GET_LOCATION_TIMEOUT:
                    HUD.hudShowError(`GetLastKnownLocation: GET_LOCATION_TIMEOUT, ${error.message}`)
                    break
                default:
                    HUD.hudShowError(`GetLastKnownLocation: ${error.code}, ${error.message}`)
            }
        }
    }

    async onPressPermissionsTest() {
        await (await sdkClient).navigator.push('/permissions')
    }

    render() {
        return (
            <NavBarPage
                title={TXT('nav-title')}
                backButtonType="cancel"
                menuItems={[
                    {
                        title: 'Edit',
                        callback: () => this.onPressTextNavBarMenu(),
                    },
                    {
                        icon: Assets.current.navbar.close,
                        callback: () => this.onPressTextNavBarMenu(),
                    },
                ]}
                onBackButtonTapped={async () => {
                    await (await sdkClient).navigator.pop('Passing back data')
                }}
            >
                <ScrollView>
                    <View style={styles.container}>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Push (normal)"
                                onPress={() => this.onPressPush()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Push (modal)"
                                onPress={() => this.onPressPresent()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Config"
                                onPress={() => this.onPressConfig()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Authenticate"
                                onPress={() => this.onPressAuth()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Scanner"
                                onPress={() => this.onPressScanner()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Store test"
                                onPress={() => this.onPressStoreTest()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Components test page"
                                onPress={() => this.onPressComponentsTest()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Calendar test page"
                                onPress={() => this.onPressCalendarTest()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Test Navigate to User Profile"
                                onPress={() => this.onPressNavigateToUserProfile()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="WebView Test"
                                onPress={() => this.onPressWebView()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Image Picker"
                                onPress={() => this.onPressImagePicker()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Share App Link"
                                onPress={() => this.onPressShareAppLink()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="FileSystem Test"
                                onPress={() => this.onPressFileSystemTest()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="FilePreview Test"
                                onPress={() => this.onPressFilePreviewTest()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Get Last Known Location Test"
                                onPress={() => this.onPressGetLastKnownLocationTest()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Permissions Test"
                                onPress={() => this.onPressPermissionsTest()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Props Test"
                                onPress={() => this.onPressPropsTest()}
                            />
                        </View>
                    </View>
                </ScrollView>
            </NavBarPage>
        )
    }
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        alignItems: 'center',
        justifyContent: 'center',
    },
    button: {
        height: 80,
        display: 'flex',
        flexDirection: 'row',
    },
    solidButton: {
        marginHorizontal: 20,
        width: 160,
        height: 44,
    },
})
