import { PageComponent, StyleSheet, View } from '@seatalk-rn/sdk-components'
import { FileInfo, IFileManager } from '@seatalk-rn/sdk-filesystem'
import { IFilePreviewer } from '@seatalk-rn/sdk-preview'
import { HUD, NavBarPage, SolidButton } from '@seatalk-rn/sdk-widgets'
import React from 'react'
import { Picker, ScrollView } from 'react-native'
import { sdkClient } from '../client'

export class FilePreviewTestPage extends PageComponent {

    private logger = sdkClient.then(client => client.buildLogger('FilePreviewTestPage'))
    state = { fileType: 'images' }

    private static get fileManager(): Promise<IFileManager> {
        return sdkClient.then(client => client.fileSystem.fileManager)
    }

    private static get filePreviewer(): Promise<IFilePreviewer> {
        return sdkClient.then(client => client.filePreview.filePreviewer)
    }

    getPageName(): string {
        return 'FilePreviewTestPage'
    }

    updateFileType = (fileType: any) => {
        this.setState({ fileType: fileType })
    }

    async onPressPreviewFile(showMenu: boolean, selectedIndex: number) {
        const filePreviewer = await FilePreviewTestPage.filePreviewer

        if (!this.state.fileType) {
            HUD.hudShowError('Please pick a filetype')
        }

        try {
            if (this.state.fileType == 'images') {
                const localImages = await this.prepareImages()
                await filePreviewer.previewImages(localImages, selectedIndex, { showMenu: showMenu })
            } else if (this.state.fileType == 'file') {
                const localFileInfo = await this.prepareFile()
                await filePreviewer.previewFile(localFileInfo, { showMenu: showMenu })
            }
        } catch (e) {
            HUD.hudShowError(`${e}`);
            (await this.logger).error(`${e}`)
        }
    }

    // testID: 1 => Preview images with index out of bound
    // testID: 2 => Preview file without valid file
    // testID: 3 => Preview images with no images supplied
    // testID: 4 => Preview images with -ve index

    async onPressStressTest(testID: number) {
        const filePreviewer = await FilePreviewTestPage.filePreviewer

        try {
            if (testID == 1) {
                let localImages = await this.prepareImages()
                await filePreviewer.previewImages(localImages, 10, { showMenu: true })
            } else if (testID == 2) {
                const unknownFileInfo: FileInfo = { type: 'document', path: 'random' }
                await filePreviewer.previewFile(unknownFileInfo, { showMenu: true })
            } else if (testID == 3) {
                await filePreviewer.previewImages([], 10, { showMenu: true })
            } else if (testID == 4) {
                let localImages = await this.prepareImages()
                await filePreviewer.previewImages(localImages, -1, { showMenu: true })
            }
        } catch (e) {
            HUD.hudShowError(`${e}`);
            (await this.logger).error(`${e}`)
        }
    }

    async prepareFile(): Promise<FileInfo> {
        const fileManager = await FilePreviewTestPage.fileManager
        const fileInfo: FileInfo = { type: 'document', path: 'preview/dog.pdf' }
        await fileManager.mkDir({ type: 'document', path: 'preview' })
        const exists = await fileManager.exists(fileInfo)
        if (exists) {
            console.log('Found pdf')
            return fileInfo
        } else {
            const result = await fileManager.downloadFile(
                'https://www.pennfoster.edu/~/media/Files/PDF/SampleLessons/448-Animal%20Care%20Dogs%20Certificate.ashx',
                fileInfo,
                undefined
            )
            if (result == 200) {
                console.log('Downloaded pdf')
                return fileInfo
            } else {
                throw Error('Download failed')
            }
        }
    }

    async prepareImages(): Promise<FileInfo[]> {
        const fileManager = await FilePreviewTestPage.fileManager
        const files: FileInfo[] = [
            { type: 'document', path: 'preview/dog.jpg' },
            { type: 'document', path: 'preview/cat.jpg' },
            { type: 'document', path: 'preview/lab.png' },
            { type: 'document', path: 'preview/elephant.jpeg' },
            { type: 'document', path: 'preview/unknown.png' }
        ]
        await fileManager.mkDir({ type: 'document', path: 'preview' })
        const filePromises = files.map(async file => {
            const exists = await fileManager.exists(file)
            if (exists) {
                console.log(`Found image ${JSON.stringify(file)}`)
                return file
            } else {
                if (file.path.endsWith('unknown.png')) {
                    return file
                }

                const downloadURL: string = this.getDownloadLink(file)
                const result = await fileManager.downloadFile(downloadURL, file, undefined)
                if (result == 200) {
                    console.log(`Downloaded image ${JSON.stringify(file)}`)
                    return file
                } else {
                    throw Error('Download failed')
                }
            }
        })

        return Promise.all(filePromises)
    }

    render() {
        return (
            <NavBarPage title="FileSystem Test" backButtonType="back">
                <ScrollView>
                    <View style={styles.container}>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Preview"
                                onPress={() => this.onPressPreviewFile(true, 0)}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Preview without menu"
                                onPress={() => this.onPressPreviewFile(false, 3)}
                            />
                        </View>
                    </View>
                    <Picker
                        style={{ display: 'flex' }}
                        selectedValue={this.state.fileType}
                        onValueChange={this.updateFileType}
                    >
                        <Picker.Item label="images" value="images"/>
                        <Picker.Item label="file" value="file"/>
                    </Picker>
                    <View style={styles.container}>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Preview out of bounds"
                                onPress={() => this.onPressStressTest(1)}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Preview without file"
                                onPress={() => this.onPressStressTest(2)}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Preview images with none"
                                onPress={() => this.onPressStressTest(3)}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Preview images -ve index"
                                onPress={() => this.onPressStressTest(4)}
                            />
                        </View>
                    </View>
                </ScrollView>
            </NavBarPage>
        )
    }

    private getDownloadLink = (fileInfo: FileInfo) => {
        if (fileInfo.path.endsWith('dog.jpg')) {
            return 'https://unsplash.com/photos/edbhAWFxnTA/download?force=true'
        } else if (fileInfo.path.endsWith('lab.png')) {
            return 'https://freepngimg.com/save/56843-labrador-download-free-download-png-hd/400x400'
        } else if (fileInfo.path.endsWith('cat.jpg')) {
            return 'https://unsplash.com/photos/Tn8DLxwuDMA/download?force=true&w=640'
        } else if (fileInfo.path.endsWith('elephant.jpeg')) {
            return 'https://freepngimg.com/convert-png/13792-elephant-free-download-png'
        } else {
            return ''
        }
    }
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        alignItems: 'center',
        justifyContent: 'center'
    },
    button: {
        height: 80,
        display: 'flex',
        flexDirection: 'row'
    },
    solidButton: {
        marginHorizontal: 20,
        width: 160,
        height: 44
    }
})
