import { PageComponent, StyleSheet, View } from '@seatalk-rn/sdk-components'
import { IFileManager } from '@seatalk-rn/sdk-filesystem'
import { Alert, HUD, ImageInfo, NavBarPage, SolidButton } from '@seatalk-rn/sdk-widgets'
import React from 'react'
import { ScrollView } from 'react-native'
import { sdkClient } from '../client'


export class FileSystemTestPage extends PageComponent {
    private logger = sdkClient.then(client => client.buildLogger('FilePreviewTestPage'))

    private static get fileManager(): Promise<IFileManager> {
        return sdkClient.then(client => client.fileSystem.fileManager)
    }

    getPageName(): string {
        return 'FileSystemTestPage'
    }

    async onPressReadFile() {
        let fileName: string = ''
        let fileContent: string = ''

        await Alert.show(
            'Enter the file path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        fileName = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/seatalk',
            }
        )

        if (fileName.length == 0) {
            HUD.hudShowSuccess('Empty filename, skipping operation')
            return
        }

        const fileManager = await FileSystemTestPage.fileManager
        try {
            fileContent = await fileManager.readFile({ type: 'document', path: fileName })
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError('Error when reading file')
        }

        if (fileContent) {
            if (fileContent.length <= 200) {
                await Alert.show('File contents', fileContent, { title: 'OK', style: 'normal' })
            } else {
                await Alert.show('File content too long, showing first 200 chars', fileContent.substr(0, 200), {
                    title: 'OK',
                    style: 'normal',
                })
            }
        }
    }

    async onPressWriteFile() {
        let fileName: string = ''
        let fileContent: string = ''

        await Alert.show(
            'Enter the file path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        fileName = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/seatalk',
            }
        )

        await Alert.show(
            'Enter the contents',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        fileContent = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g This is my file content',
            }
        )

        if (fileName.length == 0) {
            HUD.hudShowSuccess('Empty filename, skipping operation')
            return
        }

        const fileManager = await FileSystemTestPage.fileManager
        try {
            await fileManager.writeFile({ type: 'document', path: fileName }, fileContent)
            HUD.hudShowSuccess(`Successfully wrote to file ${fileName}`)
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError(`Error when writing file: ${error}`)
        }
    }

    async onPressReadDir() {
        let dirName: string = ''
        let dirContent: string[] = []

        await Alert.show(
            'Enter the directory path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        dirName = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/seatalk',
            }
        )

        const fileManager = await FileSystemTestPage.fileManager
        try {
            dirContent = await fileManager.readDir({ type: 'document', path: dirName })
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError('Error when reading directory')
        }

        if (dirContent.length > 0) {
            let files = dirContent
                .map((content, index) => `${index + 1}. ${content}`)
                .reduce((prev, curr) => `${prev} \n ${curr}`)

            await Alert.show('Directory contents', files, { title: 'OK', style: 'normal' })
        } else {
            HUD.hudShowSuccess('No items found in this directory')
        }
    }

    async onPressMakeDir() {
        let dirName: string = ''

        await Alert.show(
            'Enter the directory path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        dirName = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/seatalk',
            }
        )

        if (dirName.length == 0) {
            HUD.hudShowSuccess('Empty input, skipping operation')
            return
        }

        const fileManager = await FileSystemTestPage.fileManager
        try {
            await fileManager.mkDir({ type: 'document', path: dirName })
            HUD.hudShowSuccess(`Made directory at ${dirName}`)
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError('Error when making directory')
        }
    }

    async onPressDeleteItem() {
        let itemName = await this.promptItemName()

        if (itemName.length == 0) {
            HUD.hudShowSuccess('Empty input, skipping operation')
            return
        }

        const fileManager = await FileSystemTestPage.fileManager
        try {
            await fileManager.remove({ type: 'document', path: itemName })
            HUD.hudShowSuccess(`Deleted item at path ${itemName}`)
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError('Error when deleting item')
        }
    }

    async onPressCheckItem() {
        const itemName = await this.promptItemName()

        if (itemName.length == 0) {
            HUD.hudShowSuccess('Empty input, skipping operation')
            return
        }

        const fileManager = await FileSystemTestPage.fileManager
        try {
            const result = await fileManager.exists({ type: 'document', path: itemName })
            if (result) {
                HUD.hudShowSuccess(`Item exists at ${itemName}`)
            } else {
                HUD.hudShowSuccess(`No item exists at ${itemName}`)
            }
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError('Error when deleting item')
        }
    }

    async onPressDownloadFile() {
        let url: string = ''
        let destPath: string = ''

        await Alert.show(
            'Enter the remote url',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        url = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g http://your_file_url',
            }
        )

        await Alert.show(
            'Enter the destination path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        destPath = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/seatalk',
            }
        )

        if (url.length == 0 || destPath.length == 0) {
            HUD.hudShowSuccess('Empty input, skipping operation')
            return
        }

        const fileManager = await FileSystemTestPage.fileManager
        try {
            const result = await fileManager.downloadFile(url, { type: 'cache', path: destPath }, undefined)
            if (result == 200) {
                HUD.hudShowSuccess(`Item downloaded to ${destPath}`)
            } else {
                HUD.hudShowSuccess(`Error while downloading from ${url} status code: ${result}`)
            }
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError(`Error when downloading item ${error}`)
        }
    }

    async onPressDownloadImage() {
        let url: string = ''
        let destPath: string = ''

        await Alert.show(
            'Enter the remote image url',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        url = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g http://your_file_url',
            }
        )

        await Alert.show(
            'Enter the image path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        destPath = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/myImage.jpg',
            }
        )

        if (url.length == 0 || destPath.length == 0) {
            HUD.hudShowSuccess('Empty input, skipping operation')
            return
        }

        const client = await sdkClient
        const fileManager = client.fileSystem.fileManager
        try {
            const result = await fileManager.downloadFile(url, { type: 'cache', path: destPath }, undefined)
            if (result == 200) {
                HUD.hudShowSuccess(`Image downloaded to ${destPath}`)
                const imageInfo: ImageInfo = {
                    fileUri: await fileManager.getFullURI({ type: 'cache', path: destPath }),
                    width: 400,
                    height: 300,
                }
                try {
                    await client.navigator.push('/imageviewer', { images: [imageInfo] })
                } catch (error) {
                    (await this.logger).error(`Diff error ${error}`)
                    HUD.hudShowError('Error when downloading item')
                }
            } else {
                HUD.hudShowSuccess(`Error while downloading from ${url} status code: ${result}`)
            }
        } catch (error) {
            (await this.logger).error(`Diff error ${error}`)
            HUD.hudShowError(`Error when downloading item ${error}`)
        }
    }

    async onPressViewImage() {
        let filePath: string = ''

        await Alert.show(
            'Enter the image url',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        filePath = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g /home/test.png',
            }
        )

        if (filePath.length == 0) {
            HUD.hudShowSuccess('Empty input, skipping operation')
            return
        }

        const client = await sdkClient
        const fileManager = client.fileSystem.fileManager
        const imageInfo: ImageInfo = {
            fileUri: await fileManager.getFullURI({ type: 'document', path: filePath }),
            width: 400,
            height: 300,
        }
        await client.navigator.push('/imageviewer', { images: [imageInfo] })
    }

    render() {
        return (
            <NavBarPage title="FileSystem Test" backButtonType="back">
                <ScrollView>
                    <View style={styles.container}>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Read file"
                                onPress={() => this.onPressReadFile()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Write File"
                                onPress={() => this.onPressWriteFile()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Make Directory"
                                onPress={() => this.onPressMakeDir()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Read Directory"
                                onPress={() => this.onPressReadDir()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Check Item exists"
                                onPress={() => this.onPressCheckItem()}
                            />
                            <SolidButton
                                critical={true}
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Delete item"
                                onPress={() => this.onPressDeleteItem()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Download File"
                                onPress={() => this.onPressDownloadFile()}
                            />
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="Download Image"
                                onPress={() => this.onPressDownloadImage()}
                            />
                        </View>
                        <View style={styles.button}>
                            <SolidButton
                                backgroundStyle={styles.solidButton}
                                enabled={true}
                                title="View Downloaded image"
                                onPress={() => this.onPressViewImage()}
                            />
                        </View>
                    </View>
                </ScrollView>
            </NavBarPage>
        )
    }

    private async promptItemName() {
        let itemName: string = ''

        await Alert.show(
            'Enter the item path',
            '',
            {
                title: 'Confirm',
                style: 'normal',
                callback: input => {
                    if (input) {
                        itemName = input
                    }
                },
            },
            {
                title: 'Cancel',
                style: 'critical',
                callback: () => {
                    HUD.hudShowSuccess('Cancelled')
                    return
                },
            },
            {
                placeholder: 'e.g home/seatalk',
            }
        )
        return itemName
    }
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        alignItems: 'center',
        justifyContent: 'center',
    },
    button: {
        height: 80,
        display: 'flex',
        flexDirection: 'row',
    },
    solidButton: {
        marginHorizontal: 20,
        width: 160,
        height: 44,
    },
})
