import { PageComponent, StyleSheet, Text } from '@seatalk-rn/sdk-components'
import { getLastKnownLocation, Location } from '@seatalk-rn/sdk-location'
import { hasPermissions, PermissionInfo, PermissionType } from '@seatalk-rn/sdk-permission'
import { HUD, NavBarPage, SolidButton } from '@seatalk-rn/sdk-widgets'
import React, { ReactNode } from 'react'
import { ScrollView } from 'react-native'

interface State {
    permissions: PermissionInfo[]
    location?: Location
}

export class PermissionsTestPage extends PageComponent<{}, State> {
    private static locationPermissions: PermissionType[] = ['location', 'preciseLocation']
    state: State = { permissions: [] }

    getPageName(): string {
        return 'PermissionsTestPage'
    }

    async pageBecameReady() {
        super.pageBecameReady()
        await this.updateLocation()
        await this.updatePermissions()
    }

    async pageBecameUnhidden() {
        super.pageBecameUnhidden()
        await this.updatePermissions()
        if (this.allLocationPermissionsAreGranted()) {
            await this.updateLocation()
        }
    }

    render(): ReactNode {
        return (
            <NavBarPage title="Permissions Test" backButtonType="back">
                <ScrollView style={{ paddingHorizontal: 12 }}>
                    {this.state.permissions.map(({ permission, granted }) => (
                        <Text key={permission}>{`${permission}: ${granted ? 'Granted' : 'Not Granted'}`}</Text>
                    ))}
                    {this.renderLocation()}
                    <SolidButton
                        backgroundStyle={styles.solidButton}
                        title="Request Location"
                        enabled={true}
                        onPress={async () => {
                            await this.updateLocation()
                            await this.updatePermissions()
                        }}
                    />
                </ScrollView>
            </NavBarPage>
        )
    }

    private renderLocation(): ReactNode {
        const { location } = this.state
        const text = location ? `Latitude: ${location.latitude}, Longitude: ${location.longitude}` : 'Location Unknown'
        return <Text>{text}</Text>
    }

    private async updateLocation(): Promise<void> {
        try {
            const lastKnownLocation = await getLastKnownLocation()
            HUD.hudShowSuccess('Location updated')
            this.setState(prevState => ({ ...prevState, location: lastKnownLocation }))
        } catch (e) {
            HUD.hudShowError(e.toString())
        }
    }

    private async updatePermissions(): Promise<void> {
        try {
            const permissions = await hasPermissions(PermissionsTestPage.locationPermissions)
            this.setState(prevState => ({ ...prevState, permissions }))
        } catch (e) {
            HUD.hudShowError(e.toString())
        }
    }

    private allLocationPermissionsAreGranted(): boolean {
        return PermissionsTestPage.locationPermissions.every(
            permission => !!this.state.permissions.find(info => info.permission === permission)?.granted
        )
    }
}

const styles = StyleSheet.create({
    solidButton: {
        marginTop: 8,
        alignSelf: 'center',
        width: 160,
        height: 44,
    },
})
