const chalk = require("chalk");
const path = require("path");
const ejs = require("ejs");
const ora = require("ora");
const fs = require("fs-extra");

const loading = ora("start loading");

// terminal show red color error text output
function outputError(str) {
  // Add ANSI escape codes to display text in red.
  return process.stdout.write(chalk.red(`\r\n [ERR] ${str} \r\n \r\n`));
}

// terminal show cyan color options text output
function outputOptions(str) {
  return process.stdout.write(chalk.cyan(`\r\n [option] ${str} \r\n \r\n`));
}

// terminal show blue color tips text output
function outputTips(str) {
  return process.stdout.write(chalk.blue(`\r\n [TIPS] ${str} \r\n \r\n`));
}

// terminal show green color success text output
function outputSuccessText(str) {
  return process.stdout.write(chalk.green(`\r\n [DONE] ${str} \r\n \r\n`));
}

// terminal show yellow color warming text output
function outputWarmingText(str) {
  return process.stdout.write(chalk.yellow(`\r\n [WARM] ${str} \r\n \r\n`));
}

// write file or dir
function write(fileName, sourceDir, targetDir, content) {
  const targetPath = path.join(targetDir, fileName);
  if (content) {
    setLoading({ text: targetPath });
    fs.writeFileSync(targetPath, content);
  } else {
    copy(path.join(sourceDir, fileName), targetPath);
  }
}

// copy file or dir
function copy(src, dest) {
  const stat = fs.statSync(src);
  if (stat.isDirectory()) {
    copyDir(src, dest);
  } else {
    setLoading({ text: dest });
    fs.copyFileSync(src, dest);
  }
}

function copyDir(srcDir, destDir) {
  fs.mkdirSync(destDir, { recursive: true });
  for (const file of fs.readdirSync(srcDir)) {
    const srcFile = path.resolve(srcDir, file);
    const destFile = path.resolve(destDir, file);
    copy(srcFile, destFile);
  }
}

/**
 * use ejs render files
 * @param {*} renderFilesList
 *
 * Array: renderFilesList<renderFile>
 *
 * Object: renderFile
 * {
 *    name: 'fileName',
 *    data: { [key]: [value] },
 *    sourcePath: 'pathStr',
 *    targetPath: 'pathStr',
 * }
 */
function renderFiles(renderFilesList) {
  if (!renderFilesList || !renderFilesList.length) {
    return;
  }
  renderFilesList.forEach(({ name, sourcePath, targetPath, data }) => {
    const content = fs.readFileSync(path.join(sourcePath, name), "utf-8");
    const renderContent = ejs.render(content, { data });
    write(name, sourcePath, targetPath, renderContent);
  });
}

function setLoading(options) {
  if (!options) {
    loading.stop();
    return;
  }
  loading.start();
  if (options.text) {
    loading.text = options.text;
  }
  if (options.color) {
    loading.color = options.color;
  }
}

module.exports = {
  outputError,
  outputTips,
  outputOptions,
  outputSuccessText,
  outputWarmingText,
  copy,
  setLoading,
  write,
  renderFiles,
};
